(defvar inhibit-rename-screen nil "List of screens not to rename on select")
;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
;;; This uses an undocumented feature that SMK claims crashes Epoch.
;;; However, I "fixed" his code, and tested with the hook function being
;;; garbage-collect, and that seemed to work just fine.

(defun amc-set-name (win)
  (let*
    (
      (buff (window-buffer win))
      (name (or (and buff (buffer-name buff)) "# No-Name #") )
      (scr (epoch::screen-of-window win))
    )
    (if
      (and
	buff
	(buffer-file-name buff)
	(not (memq scr inhibit-rename-screen))
      )
      (progn (epoch::title name scr) (epoch::icon-name name scr))
    )
  )
)
(setq set-window-buffer-hook 'amc-set-name)
;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
(defun x-switch-to-buffer (buffer &optional create)
  "Switch to a window with the buffer displayed.
If no such window, a new screen is created and the buffer displayed there.
Optional argument create indicates that the buffer is to be created if it
doesn't already exist."
  (setq w (epoch::get-buffer-window buffer))
  (if w
    ;; then - there's a window&screen with buffer
    (progn
      (select-screen (epoch::screen-of-window w)) ;to keep things up to date
      (select-window w)			;pick the window
      (mapraised-screen)			; make sure it's mapped
    )
    ;; else - no window - make new screen and go there
    (progn
      (setq newbuff (get-buffer buffer)) ; already exist?
      ; if not, and create set, and it was a string
      (if (and (not newbuff) create (stringp buffer))
	; then make a new buffer
	(setq newbuff (generate-new-buffer buffer))
      )
      (if newbuff
	(let ( (name (or (buffer-name newbuff) "Edit")) )
	  (raise-screen
	    (select-screen
	      (create-screen newbuff
		(list (cons 'title name) (cons 'icon-name name))
	      )
	    )
	  )
	)
      )
    )
  )					; if buffer
)

(defun x-find-file (filename)
  "Edit file FILENAME.
Switch to a window/screen visiting file FILENAME,
creating one if none already exists."
  (interactive "FX Find File: ")
  (x-switch-to-buffer (find-file-noselect filename))
)

(defun x-find-buffer (buffer)
  "Edit buffer BUFFER.
Switch to a window/screen with the buffer already present,
or create a new screen if none."
  (interactive "BX Find Buffer: ")
  (x-switch-to-buffer buffer)
)
;;; --------------------------------------------------------------------------
(defun kill-entire-line ()
  (interactive)
  (save-excursion
    (beginning-of-line)
    (setq start (point))
    (beginning-of-line 2)
    (kill-region start (point))
  )
)

(defun kill-start-of-line ()
  (interactive)
  (let ( (there (point)) )
    (beginning-of-line)
    (kill-region (point) there)
  )
)

(defun join-lines ()
  (interactive)
  (save-excursion
    (end-of-line)			;go the end
    (delete-char 1)			;get rid of the newline
    (just-one-space)			;remove all but one space
  )
)
;;; ------------------------------------------------------------------------
(defun count-screen-lines (direction)	; t for up, nil for down
  (let
    (
      (lc -1)
      (inc (if direction -1 1))
    )
    (save-excursion
      (while (and (pos-visible-in-window-p) (/= (vertical-motion inc) 0) )
	(setq lc (+ 1 lc))
      )
    )
    (max lc 0)				;bound it below
  )
)

(defun move-page (count)
  (interactive "N")
  (setq count (prefix-numeric-value count))
  (let ( (start-x (count-screen-lines t)) )
    (while (/= count 0)
      (if (> count 0)
	(progn (scroll-up nil) (setq count (- count 1)))
	(progn (scroll-down nil) (setq count (+ count 1)))
      )
    )
    (move-to-window-line 0)
    (vertical-motion start-x)
  )
)

(defun page-up (count) (interactive "p") (move-page (- count)))
(defun page-down (count) (interactive "p") (move-page count))

(defun scroll-up-one () (interactive) (scroll-up 1) (forward-line 1) )

(defun scroll-down-one () (interactive) (scroll-down 1) (forward-line -1) )

;;; ----------------------------------------------------------------------
(defun scroll-process-output (process data)
;  (save-excursion
;    (set-buffer (process-buffer process))
;    (goto-char (process-mark process))
;    (insert-before-markers data)
;  )
  (princ data (process-mark process))
  (let ( (win (epoch::get-buffer-window (process-buffer process))) )
    (if (not (eq win (selected-window)))
      (set-window-point win (process-mark process))
    )
    (setq win-debug win)		;debug code
  )
)

(defun scroll-this-process-buffer ()
  (interactive)
  (set-process-filter
    (get-buffer-process (current-buffer)) 'scroll-process-output
  )
  (if (not epoch::global-update) (setq epoch::global-update t))
  (epoch::set-update t)
)
;;; --------------------------------------------------------------------------
;;; MH support
;;;
(defvar mh-screen nil)
(defun mh-set-up () (interactive)
  (if (and (screenp mh-screen) (get-screen-id mh-screen))
    (mapraised-screen mh-screen)
    (setq mh-screen
      (create-screen "*scratch*"
	'((geometry . "80x45-0-80") (icon-name . "MH-Mailer")
	  (title . "MH-Mailer")
	)
      )
    )
  )
  (or
    (memq mh-screen inhibit-rename-screen)
    (setq inhibit-rename-screen (cons mh-screen inhibit-rename-screen))
  )
  (on-map-do mh-screen
    (function (lambda (s) 
	(epoch::icon-name "MH-Mailer" mh-screen)
	(epoch::title "MH-Mailer" mh-screen)
	(select-screen mh-screen)
	(mh-rmail)
      ))))
;;; ------------------------------------------------------------------------
;(fset 'old-create-screen (symbol-function 'create-screen))
;(defun create-screen (&optional buffer alist)
;  (old-create-screen buffer
;    (cons (cons 'geometry (init-screen-geometry)) alist)
;  )
;)
;;; - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
(global-unset-key "\C-z")

(global-set-key "\C-zf" 'x-find-file)
(global-set-key "\C-zb" 'x-find-buffer)
(global-set-key "\C-z\C-f" 'x-find-file)
(global-set-key "\C-z\C-b" 'x-find-buffer)
(global-set-key "\C-z0" 'delete-screen)
(global-set-key "\C-za" (definteractive (title (title)) (icon-name (icon-name))))

(global-set-key "\M-g" 'goto-line)

;; line killing
(global-set-key "\C-l" 'kill-line)
(global-set-key "\C-k" 'kill-entire-line)
(global-set-key "\C-j" 'kill-start-of-line)

;(epoch::rebind-key "j" with-alt "\C-^A_j")
;(global-set-key "\C-^A_j" 'join-lines)

(global-unset-key "\C-x\C-u")		; what a dumb binding that was.

(global-set-key "\C-zi" (definteractive (iconify-screen)))
(global-set-key "\C-ze" 'display-event-status)

(global-set-key "\M-\C-l" (definteractive
    (add-change-log-entry nil "ChangeLog")))
