;;; [cjl]
;;; uglify.el -- this file defines the function uglify-current-buffer and
;;;              supporting code, to reformat lisp code to match some
;;;              "standards".
;;;
(defun uglify-current-buffer ()
  "Reformat lisp code in the current buffer to 'standard'"
  (interactive)
  (lisp-mode)
  (munge-parens)
  (indent-sexps))
;;
;; munge-parens -- collect any stray )'s separated from code by white-space
;;                 and/or newlines.
;;
(defun munge-parens ()
  (interactive)
  (goto-char 1)
  (while (re-search-forward srch nil t)
    (delete-region (+ (match-beginning 0) 1) (- (match-end 0) 1))
    (previous-line 1)))
;;
;; regular expression used as search pattern.
;;
(setq srch  "[)]\\(\t\\| \\)*\n\\(\n\\|\t\\| \\)*[)]" )
;;
;; indent-sexps -- cycle through s-expressions in buffer, and reformat using
;;                 indent-region.
;;
(defun indent-sexps ()
  (interactive)
  (lisp-mode)
  (goto-char 1)
  (let ( (last (point))
  (while (not (= last (next-sexp)))
    (indent-region (prev-sexp) (next-sexp) nil)
    (setq last (next-sexp))))))
;;
;; next-sexp -- move to end of next s-expression, and return its position.
;;
(defun next-sexp ()
  (interactive)
  (forward-sexp)
  (point))
;;
;; prev-sexp -- move to beginning of current s-expression, and return position.
;;
(defun prev-sexp ()
  (interactive)
  (backward-sexp)
  (point))





