/*
 * [cjl]
 * Support code for all primitives related to attributes
 */

#include "config.h"
#include "lisp.h"
#include "attrib.h"		/* type definition for attribute */
#include "dispextern.h"
#include "buffer.h"
#include "screen.h"
#include "screenW.h"
#include "screenX.h"

#define MAX_ATTRIBUTE 256
struct Lisp_Attribute Attribute_table[MAX_ATTRIBUTES];

void create_table ()
{
  Lisp_Object val;
  int i;

  for (i = 0; i <= MAX_ATTRIBUTES; i+=1)
    {
      val = Fmake_vector (make_number((sizeof (struct Lisp_Attribute) -
				       sizeof (struct Lisp_Vector) +
				       sizeof (Lisp_Object))
				      / sizeof (Lisp_Object)), Qnil);
      XSETTYPE (val, Lisp_Attribute);
      Attribute_table[i] = XATTRIBUTE(val);
      /* do initialization here, so every entry contains non-junk */
      
    }
}

DEFUN ("epoch::foreground",Fforeground,Sforeground,1,2,0,
       "(epoch::foreground index [screenID]) returns the foreground color\n\
defined in this entry of the attribute table (optionally for the selected\n\
screen).")
      (index, screen) Lisp_Object index, screen;
{
  CHECK_NUMBER (index,0);
  if ((index < 0) || (index > MAX_ATTRIBUTES - 1))
    Fsignal(Qerror,Fcons(build_string("index out of range"),Fcons(index,Qnil)));
  return XATTRIBUTE(Attribute_table[index])->foreground;
}

DEFUN ("epoch::background",Fbackground,Sbackground,1,2,0,
       "(epoch::background index [screenID]) returns the background color\n\
defined in this entry of the attribute table (optionally for the selected\n\
screen).")
      (index, screen) Lisp_Object index,screen;
{
  CHECK_NUMBER (index,0);
  if ((index < 0) || (index > MAX_ATTRIBUTES - 1))
    Fsignal(Qerror,Fcons(build_string("index out of range"),Fcons(index,Qnil)));
  return XATTRIBUTE(Attribute_table[index])->background;
}

DEFUN ("epoch::cursor_foreground",Fcursor_foreground,Scursor_foreground,1,2,0,
       "(epoch::cursor_foreground index [screenID]) returns the cursor \n\
foreground color defined in this entry of the attribute table (optionally\n\
for the selected screen).")
      (index, screen) Lisp_Object index,screen;
{
  CHECK_NUMBER (index,0);
  if ((index < 0) || (index > MAX_ATTRIBUTES - 1))
    Fsignal(Qerror,Fcons(build_string("index out of range"),Fcons(index,Qnil)));
  return XATTRIBUTE(Attribute_table[index])->c_fore;
}

DEFUN ("epoch::cursor_background",Fcursor_background,Scursor_background,1,2,0,
       "(epoch::cursor_background index [screenID]) returns the cursor\n\
background color defined in this entry of the attribute table (optionally\n\
for the selected screen's attribute table).")
       (index, screen) Lisp_Object index,screen;
{
  CHECK_NUMBER (index,0);
  if ((index < 0) || (index > MAX_ATTRIBUTES - 1))
    Fsignal(Qerror,Fcons(build_string("index out of range"),Fcons(index,Qnil)));
  return XATTRIBUTE(Attribute_table[index])->c_back;
}

DEFUN ("epoch::stipple",Fstipple,Sstipple,1,2,0,
       "(epoch::stipple index [screenID]) returns the stipple pattern for\n\
the indicated entry in the attribute table (optionally for the selected\n\
screen's attribute table).")
       (index, screen) Lisp_Object index,screen;
{
  CHECK_NUMBER (index,0);
  if ((index < 0) || (index > MAX_ATTRIBUTES - 1))
    Fsignal(Qerror,Fcons(build_string("index out of range"),Fcons(index,Qnil)));
  return XATTRIBUTE(Attribute_table[index])->c_back;
}
    
DEFUN ("epoch::set-foreground",Fset_foreground,Sset_foreground,2,3,0,
       "(epoch::set-foreground index foreground [screen ID])\n\
sets the foreground color to a new value for the indicated entry in the\n\
attribute table.")
      (index, foreground, screen) Lisp_Object index, foreground, screen;
{
  CHECK_NUMBER (index,0);
  if ((index < 0) || (index > MAX_ATTRIBUTES - 1))
    Fsignal(Qerror,Fcons(build_string("index out of range"),Fcons(index,Qnil)));
}


Lisp_Object Qattributep;

void
syms_of_attribute()
{
  Qattributep = intern ("attributep"); staticpro(&Qattributep);
  defsubr(&Sforeground);
  defsubr(&Sbackground);
  defsubr(&Scursor_fore);
  defsubr(&Scursor_back);
  defsubr(&Sstipple);

}







